/*
Daybo Logic Utilities Library
Copyright (C) 2000-2005 Daybo Logic.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the project nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: checksum.c
  Author: David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  Title: Very simple checksum routines
  Version: 1.01
*/
#include <stddef.h>
#include <stdio.h>
#ifdef HDRSTOP
#  pragma hdrstop
#endif /*HDRSTOP*/

#include "dlstdinc.h"
#include "dlutils.h"
/*-------------------------------------------------------------------------*/
WORD dlutils_CalcChk16(
  const void *RawData,
  const size_t SizeOfData
)
{
  WORD genChk16 = 0; /* Generated 16-bit checksum */
  size_t offset;
  const BYTE *castedPtr = (const BYTE*)RawData;

  if (!castedPtr || !SizeOfData) return genChk16; /* Safety */

  for ( offset = 0U; offset < SizeOfData; offset++ ) {
    /* Add value of byte to total */
    genChk16 += *((const BYTE*)(castedPtr + offset));
  }

  return genChk16; /* Give 16-bit checksum to caller */
}
/*-------------------------------------------------------------------------*/
DWORD dlutils_CalcChk32(
  const void *RawData,
  const size_t SizeOfData
)
{
  DWORD genChk32 = 0; /* Generated 32-bit checksum */
  size_t offset;
  const BYTE *castedPtr = (const BYTE*)RawData;

  if (!castedPtr || !SizeOfData) return genChk32; /* Safety */

  for ( offset = 0U; offset < SizeOfData; offset++ )
    genChk32 += *((const BYTE*)(castedPtr + offset));

  return genChk32; /* Give 32-bit checksum to caller */
}
/*-------------------------------------------------------------------------*/
