/*
Daybo Logic Utilities Library
Copyright (C) 2000-2005 Daybo Logic.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the project nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HDRSTOP
# pragma hdrstop
#endif /*HDRSTOP*/

#include "dlstdinc.h"
#include "dlutils.h"

/* Local types */
typedef unsigned long ULONG;
/*-------------------------------------------------------------------------*/
DWORD dlutils_CalcFileChecksum32(FILE *HFile)
{
  /* Var declarations */
  ULONG lenChkFile, iLoops;
  DWORD chk32 = (DWORD)0UL;
  DLUTILS_FILE_BLOCK_SIZE fileBlockSize;
  BYTE *currentData, *remainingData;
  unsigned dataIndex;
  long int stackedFilePos;

  if (!HFile) return chk32;
  stackedFilePos = ftell(HFile);
  if (stackedFilePos == -1L)
    return chk32; /*0UL*/
  if ( !dlutils_GetFileLength(HFile, &lenChkFile) ) /* Save the length of this file */
    return chk32;

  if (!lenChkFile)
  {
    fclose(HFile);
    return chk32; /* Don't waste time, a zero length file's checksum is always 0 */
  }

  dlutils_CalcFileBlockSize(lenChkFile, &fileBlockSize, (size_t)4096U); /* Call Nigel's well-magic wonderful fast function!, buffer 4K for read loops */
  currentData = (BYTE*)malloc(fileBlockSize.BufferSize); /* Create a buffer for file I/O */

  fseek(HFile, 0, SEEK_SET); /* Reset to file begining */
  for (iLoops=0;iLoops<fileBlockSize.Loops;iLoops++)
  {
    if (fread(currentData, fileBlockSize.BufferSize, 1, HFile) < 1) /* Read data into buffer */
    {
      /* EOF should never occur - AND HAS! */
      fprintf(stderr, "Unexpected EOF during file checksummer\n");
      abort();
    }
    for ( dataIndex = 0U; dataIndex < fileBlockSize.BufferSize; dataIndex++ ) /* Go through all of the CurrentData read from the file*/
      chk32 += currentData[dataIndex]; /* Add the current byte to the total 32-bit CRC checksum */
  }
  /* Now add the remainder */
  if (fileBlockSize.Remainder)
  {
    remainingData = (BYTE*)malloc(fileBlockSize.Remainder); /* Create buffer for remainder */
    fread(remainingData, fileBlockSize.Remainder, 1, HFile); /* Put remaining data in this buffer */
  for ( dataIndex = 0U; dataIndex < fileBlockSize.Remainder; dataIndex++ ) /* go through all of the remaining data byte by byte */
    chk32 += remainingData[dataIndex]; /* Add the remainder to the total */

    free(remainingData); /* Deallocate buffer */
  }

  /* Cleanup */
  if (currentData) free(currentData);

  rewind(HFile);
  fseek(HFile, stackedFilePos, SEEK_SET); /* Restore file pos */
  return chk32; /* Give the 32-bit CRC value to the caller */
}
/*----------------------------------------------------------------------------*/
