/*
Daybo Logic Utilities Library
Copyright (C) 2000-2005 Daybo Logic.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the project nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.


Safe dynamic memory allocator.

Author: David Duncan Ross Palmer
Company: Daybo Logic
Library: Utilities
Language compliance: ANSI C (1990)
Created: 19990000
Last modified: 20050907

Serious bug fix: The SYNMEM_WANT_IGNORANCE define is needed to stop
the header converting our calls to malloc() etc. into calls back to
ourselves, this was a serious bug which caused a hidden stack overflow
which took hours to find. - OVERLORD 20th Jan 2000
*/

#include <stdlib.h>
#include "dlstdinc.h"
#ifdef HDRSTOP
# pragma hdrstop
#endif /*HDRSTOP*/

#define SYNMEM_WANT_IGNORANCE
#include "dlutils_sdynmem.h" /* Ours */

void* xmalloc(size_t size)
{
  void* ptr = malloc(size);
  if (!ptr)
  {
    /*FIXME: dlutils_FatalTrap("Out of memory");*/
    abort();
  }
  return ptr;
}

void* xrealloc(void* old, size_t size)
{
  void* ptr = realloc(old, size);
  if (!ptr)
  {
    /*FIXME: dlutils_FatalTrap("Out of memory");*/
    abort();
  }
  return ptr;
}

void* xcalloc(size_t nitems, size_t size)
{
  void* ptr = calloc(nitems, size);
  if (!ptr)
  {
    /*FIXME: dlutils_FatalTrap("Out of memory");*/
    abort();
  }
  return ptr;
}
