/*
Daybo Logic Utilities Library
Copyright (C) 2000-2005 Daybo Logic.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the project nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.


This module provides support for "out of memory" messages and
automatic program failure incase of allocation failure, so programmer's
don't have to bother with NULL returns from malloc etc.  The names of
these replacement functions are preceded by an 'x'.  Therefore
malloc() becomes xmalloc(), calloc() xcalloc(), realloc() xrealloc().

new, delete, new[] & delete[] are not supported, the operators are only
available in C++.  This library is committed to ANSI C compliance and
will ont break that compliance to keep the extensions brigade happy.
*/

#ifndef INC_DLUTILS_SDYNMEM_H
#define INC_DLUTILS_SDYNMEM_H

#ifdef __cplusplus
  extern "C" {
#endif /*__cplusplus*/
/*-------------------------------------------------------------------------*/
void *xmalloc(size_t size);
void *xrealloc(void *old, size_t size);
void *xcalloc(size_t nitems, size_t size);

/* 11th Oct 1999 : Damn!  I keep on editing the next bit of
code, but because of losses of data and resotring the backup,
it keeps disappearing again.  Please KEEP THIS DAMN CODE: */

/* Macros intended for us always goto us... */
#define XMALLOC   xmalloc
#define XREALLOC  xrealloc
#define XCALLOC   xcalloc

/* The lead programmer may define __NOSDYNMEM__ to make
normal allocation calls goto normal functions, else they
are redirected to us */
#ifdef __NOSDYNMEM__ /* Normal C library mode only? */
/* Portable macros goto C funcs */
#  define MALLOC  malloc
#  define REALLOC realloc
#  define CALLOC  calloc
#else /* We are allowed to operate normally? */
/* Redirect portable macros too ourselves */
#  define MALLOC  XMALLOC
#  define REALLOC XREALLOC
#  define CALLOC  XCALLOC
/* Redirect people that don't know about us, to us too */
#  ifndef SYNMEM_WANT_IGNORANCE
#    define malloc  XMALLOC
#    define realloc XREALLOC
#    define calloc  XCALLOC
#  endif
#endif /*__NOSDYNMEM__*/
/*-------------------------------------------------------------------------*/
#ifdef __cplusplus
  }
#endif /*__cplusplus*/

#endif /*!INC_DLUTILS_SDYNMEM_H*/
